;**************************************************************************
;*
;* FILE  fastformat.asm
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id$
;*
;* DESCRIPTION
;*   Fast format on a 1541/1571 drive.
;*
;* USAGE
;*   Verify can be disabled by poking 0 into vfy_flag. (not recommended)
;*
;*   Alt 1:
;*   ------
;*   1. Set up the device number in $ba, for example using a call to
;*      SETLFS ($ffba).
;*   2. Set up a valid format command string "N0:<name>,<id>" using
;*      a call to SETNAM ($ffbd).
;*   3. Do 'jsr TurboFormat' to start the formatting.  It will return with
;*      carry clear if formatting could be started, and carry set if not.
;*   4. When formatting has ended, status may be retrieved as usual using 
;*      the normal drive error channel.
;*
;*   Alt 2:
;*   ------
;*   1. Set up the device number in $ba, for example using a call to
;*      SETLFS ($ffba).
;*   2. Set up the desired disk name in fbuffer. (must be padded with $a0,
;*      upto 16 characters.)
;*   3. Set up the desired disk id in idbuffer. (2 characters)
;*   4. Do 'jsr DoTurboFormat' to start the formatting.  It will return with
;*      carry clear if formatting could be started, and carry set if not.
;*   5. When formatting has ended, status may be retrieved as usual using 
;*      the normal drive error channel.
;*
;* KNOWN BUGS
;*   TurboFormat and DoTurboFormat will not return carry set if writing
;*   and/or executing the drive code fails.
;*
;*
;* fastformat - a faster format for the Commodore 1541/1571 
;*
;* Copyright (c) 1995, 1996, 2002, Daniel Kahlin <daniel@kahlin.net>
;* All rights reserved.
;* 
;* Redistribution and use in source and binary forms, with or without
;* modification, are permitted provided that the following conditions
;* are met:
;* 1. Redistributions of source code must retain the above copyright
;*    notice, this list of conditions and the following disclaimer.
;* 2. Redistributions in binary form must reproduce the above copyright
;*    notice, this list of conditions and the following disclaimer in the
;*    documentation and/or other materials provided with the distribution.
;* 3. Neither the names of the copyright holders nor the names of their
;*    contributors may be used to endorse or promote products derived 
;*    from this software without specific prior written permission.
;* 
;* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
;* ``AS IS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
;* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
;* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
;* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
;* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
;* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
;* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
;* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
;* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
;* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;*
;******

	PROCESSOR 6502

ffsrc_zp	EQU	$fb	;word
ffsrcend_zp	EQU	$ac	;word
ffdest_zp	EQU	$fd	;word

;* buffers (these must be identical to fnameid in fastformat_drive.asm) *
fbuffer:
	ds.b	16
idbuffer:
	ds.b	2
vfy_flag:
	dc.b	$ff

;**************************************************************************
;*
;* TURBOFORMAT
;* IN: $bb,$bc COMMAND  $b7=LEN   (as set by SETNAM)
;* Carry=0 OK!  Carry=1 (couldn't format)
;*
;******
TurboFormat:

;* null len ? *
	lda	$b7
	beq	tf_fl1

	ldy	#0
;* check if format (N) *
	lda	($bb),y
	cmp	#"N"
	bne	tf_fl1
	iny
	cpy	$b7
	beq	tf_fl1

;* find and skip ':' *
tf_lp1:
	lda	($bb),y
	iny
	cpy	$b7
	beq	tf_fl1
	cmp	#":"	
	bne	tf_lp1


;* fill with default ($a0's) *
	ldx	#18-1	;name (16 chars) + id (2 chars)
tf_lp5:
	lda	#$a0
	sta	fbuffer,x
	dex
	bpl	tf_lp5


;* build name string *
	ldx	#0
tf_lp2:
	lda	($bb),y
	cmp	#","
	beq	tf_skp1
	sta	fbuffer,x
	inx
	iny
	cpy	$b7
	beq	tf_fl1
	cpx	#16	
	bne	tf_lp2
tf_skp1:
;* get id chars *
	ldx	#0
tf_lp3:
	iny
	cpy	$b7
	beq	tf_skp2
	lda	($bb),y
	sta	idbuffer,x
	inx
	cpx	#2	
	bne	tf_lp3

;* Do the format *
tf_skp2:
	jsr	DoTurboFormat

	clc
	rts

tf_fl1:
	sec
	rts


;**************************************************************************
;*
;* DoTurboFormat
;* Carry=0 OK!
;*
;******
DoTurboFormat:
;*** install code ***
	lda	#<fdisk
	ldy	#>fdisk
	sta	ffsrc_zp
	sty	ffsrc_zp+1
	lda	#<fdiskstart
	ldy	#>fdiskstart
	sta	ffdest_zp
	sty	ffdest_zp+1
	lda	#<fdiskend
	ldy	#>fdiskend
	sta	ffsrcend_zp
	sty	ffsrcend_zp+1
	jsr	CopyToDiskRam

;*** write name and id ***
	jsr	SendM
	lda	#"W"
	jsr	$ffa8	;CIOUT
	lda	#<fnameid
	jsr	$ffa8	;CIOUT
	lda	#>fnameid
	jsr	$ffa8	;CIOUT
	lda	#fnameidlen	;fbuffer + id + vfy_flag
	jsr	$ffa8	;CIOUT
	ldy	#$00
dtf_lp1:
	lda	fbuffer,y
	jsr	$ffa8	;CIOUT
	iny	
	cpy	#fnameidlen
	bne	dtf_lp1

	jsr	$ffae	;UNLSN


;*** do format ***
	jsr	SendM
	lda	#"E"
	jsr	$ffa8	;CIOUT
	lda	#<FormatEntry
	jsr	$ffa8	;CIOUT
	lda	#>FormatEntry
	jsr	$ffa8	;CIOUT
	jsr	$ffae	;UNLSN

	jsr	$ffe7	;CLALL

	clc
	rts

;**************************************************************************
;*
;* Send Code to drive ram
;* IN:
;*   ffsrc_zp=source in ram.
;*   ffsrcend_zp=end of source + 1
;*   ffdest_zp=destination in drive ram.
;*
;******
CopyToDiskRam:
ctdr_lp1:
	jsr	SendM
	lda	#"W"
	jsr	$ffa8	;CIOUT
	lda	ffdest_zp
	jsr	$ffa8	;CIOUT
	lda	ffdest_zp+1
	jsr	$ffa8	;CIOUT
	lda	#$1e
	jsr	$ffa8	;CIOUT
	ldy	#$00
ctdr_lp2:
	lda	(ffsrc_zp),y
	jsr	$ffa8	;CIOUT
	iny	
	cpy	#$1e
	bcc	ctdr_lp2
	jsr	$ffae	;UNLSN
	clc	
	lda	ffsrc_zp
	adc	#$1e
	sta	ffsrc_zp
	bcc	ctdr_skp1
	inc	ffsrc_zp+1
	clc	
ctdr_skp1:
	lda	ffdest_zp
	ldx	ffdest_zp+1
	adc	#$1e
	sta	ffdest_zp
	bcc	ctdr_skp2
	inc	ffdest_zp+1
ctdr_skp2:
	cpx	ffsrcend_zp+1
	bcc	ctdr_lp1
	cmp	ffsrcend_zp
	bcc	ctdr_lp1
	rts

;**************************************************************************
;*
;* Send listen to drive
;*
;******
SendListen:
	lda	$ba	;drive #
	jsr	$ffb1	;LISTEN
	lda	#$6f
	jmp	$ff93	;SECOND


;**************************************************************************
;*
;* Send  'M-' to drive
;*
;******
SendM:
	jsr	SendListen
	lda	#"M"
	jsr	$ffa8	;CIOUT
	lda	#"-"
	jmp	$ffa8	;CIOUT


;**************************************************************************
;*
;* DRIVECODE
;*
;******
fdisk:
	INCLUDE	"fastformat_drive.asm"

; eof